import * as i0 from '@angular/core';
import { Injectable, Inject, Component, Host, Input, ElementRef, Directive, ContentChildren, NgModule, SkipSelf, HostBinding, HostListener } from '@angular/core';
import { Observable, Subject, BehaviorSubject } from 'rxjs';
import * as i1$1 from '@angular/common';
import { DOCUMENT, CommonModule } from '@angular/common';
import * as i1 from '@angular/forms';
import { FormControl, FormsModule, ReactiveFormsModule, FormGroup } from '@angular/forms';
import * as i3 from 'angular-bootstrap4';
import { BsModalModule } from 'angular-bootstrap4';
import { skip, first } from 'rxjs/operators';

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TableToolsConfigService {
    constructor(document) {
        this.document = document;
        this.perPage = 25;
        this.perPageOptions = [
            { number: 10, text: '10' },
            { number: 25, text: '25' },
            { number: 50, text: '50' },
            { number: 100, text: '100' },
            { number: 200, text: '200' },
            { number: Infinity, text: 'Wszystkie' }
        ];
        this.lang = {
            first: 'Pierwsza strona',
            prev: 'Poprzednia strona',
            next: 'Następna strona',
            last: 'Ostatnia strona',
            results: 'Wyniki:',
            from: 'z',
            perPage: 'Wyników na stronę:',
            search: 'Szukaj...',
            filteredResults: 'Filtrowanie z:',
            export: 'Export',
            exportChooseColumns: 'Wybierz kolumny',
            flipSelection: 'odwróć zaznaczenie',
            exportColumnNames: 'Eksportuj nazwy kolumn',
            exportSeparator: 'Separator',
            tabulator: 'Tabulator',
            copy: 'Kopiuj',
            csv: 'CSV',
            copiedToClipboard: 'Skopiowano do schowka',
            noResults: 'Nie znaleziono żadnych wyników!'
        };
        this.scroll = true;
        this.scrollOffset = 0;
        this.exportTypes = {
            copy: {
                lang: this.lang.copy,
                callback: (data, config) => {
                    return new Observable((subscriber) => {
                        const copyCallback = (e) => {
                            // @ts-expect-error: window.clipboardData is available only in IE
                            const clipboard = e.clipboardData || this.document.defaultView.clipboardData;
                            clipboard.setData('text', data
                                .map((row) => row.join(config.separator))
                                .join('\n'));
                            e.preventDefault();
                            subscriber.next();
                            subscriber.complete();
                        };
                        this.document.addEventListener('copy', copyCallback);
                        this.document.execCommand('copy');
                        this.document.removeEventListener('copy', copyCallback);
                    });
                }
            },
            csv: {
                lang: this.lang.csv,
                parseText(txt) {
                    return '"' + txt.replace('"', '""') + '"';
                },
                callback: (data, config) => {
                    return new Observable((subscriber) => {
                        const a = this.document.createElement('a'), item = '\ufeff' + data.join('\n'), blob = new Blob([item], { type: 'text/csv;utf-8' }), url = URL.createObjectURL(blob);
                        a.setAttribute('style', 'display: none');
                        a.href = url;
                        a.download = config.fileName + '.csv';
                        this.document.body.appendChild(a);
                        a.click();
                        a.remove();
                        subscriber.next();
                        subscriber.complete();
                    });
                }
            }
        };
    }
    exportNotification(type) {
        if (type === 'copy') {
            // eslint-disable-next-line no-alert
            alert(this.lang.copiedToClipboard);
        }
    }
}
TableToolsConfigService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsConfigService, deps: [{ token: DOCUMENT }], target: i0.ɵɵFactoryTarget.Injectable });
TableToolsConfigService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsConfigService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsConfigService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: Document, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }]; } });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSelectComponent {
    constructor(tableToolsDirective) {
        this.tableToolsDirective = tableToolsDirective;
        this.formControl = new FormControl();
    }
    ngOnInit() {
        this.formControl.valueChanges.subscribe((selected) => {
            if (selected) {
                this.tableToolsDirective.tableTools.selected.select(this.item);
            }
            else {
                this.tableToolsDirective.tableTools.selected.deselect(this.item);
            }
        });
        this.tableToolsDirective.tableTools.selected.selectedChanges.subscribe((selected) => {
            const isSelected = selected.includes(this.item);
            if (isSelected !== this.formControl.value) {
                this.formControl.setValue(selected.includes(this.item), { emitEvent: false });
            }
        });
    }
}
TtSelectComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSelectComponent, deps: [{ token: TableToolsDirective, host: true }], target: i0.ɵɵFactoryTarget.Component });
TtSelectComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtSelectComponent, selector: "tt-select", inputs: { item: "item" }, ngImport: i0, template: "<!--\n  ~ Angular TableTools Plugin\n  ~  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>\n  ~  License: MIT\n  -->\n<input type=\"checkbox\" [formControl]=\"formControl\"/>\n", directives: [{ type: i1.CheckboxControlValueAccessor, selector: "input[type=checkbox][formControlName],input[type=checkbox][formControl],input[type=checkbox][ngModel]" }, { type: i1.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { type: i1.FormControlDirective, selector: "[formControl]", inputs: ["disabled", "formControl", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSelectComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-select',
                    templateUrl: './tt-select.component.html'
                }]
        }], ctorParameters: function () { return [{ type: TableToolsDirective, decorators: [{
                    type: Host
                }] }]; }, propDecorators: { item: [{
                type: Input
            }] } });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TableToolsDirective {
    constructor(elementRef, viewContainerRef) {
        this.elementRef = elementRef;
        this.viewContainerRef = viewContainerRef;
    }
    ngOnInit() {
        if (this.elementRef.nativeElement.nodeType === Node.COMMENT_NODE) {
            this.tableTools.elementRef = new ElementRef(this.viewContainerRef.element.nativeElement.previousElementSibling);
        }
        else {
            this.tableTools.elementRef = this.elementRef;
        }
    }
}
TableToolsDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsDirective, deps: [{ token: i0.ElementRef }, { token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive });
TableToolsDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.5", type: TableToolsDirective, selector: "[tableTools]", inputs: { tableTools: "tableTools" }, queries: [{ propertyName: "ttSelects", predicate: TtSelectComponent, descendants: true }], ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[tableTools]',
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.ViewContainerRef }]; }, propDecorators: { tableTools: [{
                type: Input
            }], ttSelects: [{
                type: ContentChildren,
                args: [TtSelectComponent, { descendants: true }]
            }] } });

// eslint-disable-next-line @angular-eslint/use-component-selector
class AbstractTableToolsTemplateComponent {
    constructor(config, tableToolsDirective) {
        this.tableToolsDirective = tableToolsDirective;
        this.lang = config.lang;
    }
    ngOnInit() {
        this.tableTools = this.tableToolsDirective.tableTools;
    }
}
AbstractTableToolsTemplateComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: AbstractTableToolsTemplateComponent, deps: [{ token: TableToolsConfigService }, { token: TableToolsDirective }], target: i0.ɵɵFactoryTarget.Component });
AbstractTableToolsTemplateComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: AbstractTableToolsTemplateComponent, selector: "ng-component", ngImport: i0, template: '', isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: AbstractTableToolsTemplateComponent, decorators: [{
            type: Component,
            args: [{
                    template: ''
                }]
        }], ctorParameters: function () { return [{ type: TableToolsConfigService }, { type: TableToolsDirective }]; } });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
let id = 0;
class TtExportComponent extends AbstractTableToolsTemplateComponent {
    constructor(config, tableToolsDirective, document) {
        super(config, tableToolsDirective);
        this.config = config;
        this.tableToolsDirective = tableToolsDirective;
        this.document = document;
        this.id = ++id;
        this.modal = false;
        this.columns = [];
        this.exportConfig = {
            separator: ',',
            fileName: '',
            columnNames: true
        };
        this.separators = [];
        this.exporting = false;
        this.separators = [
            { lang: ',', separator: ',' },
            { lang: ';', separator: ';' },
            { lang: this.lang.tabulator, separator: '\t' }
        ];
        this.exportTypes = config.exportTypes;
    }
    showExport() {
        const headers = this.tableToolsDirective.elementRef.nativeElement
            .querySelectorAll('table > thead > tr:last-child > th');
        this.columns = [];
        headers.forEach((h, idx) => {
            var _a;
            if (!h.classList.contains('ignore-export')) {
                this.columns.push({
                    txt: ((_a = h.textContent) === null || _a === void 0 ? void 0 : _a.trim()) || '',
                    idx,
                    checked: true
                });
            }
        });
        this.exportConfig.fileName = this.document.title;
        this.modal = true;
    }
    flipSelection() {
        this.columns.forEach((column) => {
            column.checked = !column.checked;
        });
    }
    doExport(type, config) {
        this.exporting = type;
        const indexes = [], data = [], parseText = (text) => {
            if (typeof config.parseText === 'function') {
                text = config.parseText(text);
            }
            return text;
        }, appendRow = () => {
            if (row.length) {
                data.push(row);
                row = [];
            }
        };
        let row = [];
        // get columns to export
        this.columns.forEach((column) => {
            if (column.checked) {
                indexes.push(column.idx);
                if (this.exportConfig.columnNames) {
                    row.push(parseText(column.txt));
                }
            }
        });
        appendRow();
        // grab data
        const columns = this.tableToolsDirective.elementRef.nativeElement
            .querySelectorAll('table > tbody > tr:not(.ignore-export) > td');
        let rowId = -1;
        columns.forEach((cell) => {
            var _a;
            if (indexes.includes(cell.cellIndex)) {
                if (cell.parentElement.rowIndex !== rowId) {
                    rowId = cell.parentElement.rowIndex;
                    appendRow();
                }
                row.push(parseText(((_a = cell.textContent) === null || _a === void 0 ? void 0 : _a.trim()) || ''));
            }
        });
        appendRow();
        // export
        config.callback(data, this.exportConfig).subscribe(() => {
            this.exporting = false;
            this.modal = false;
            if (typeof this.config.exportNotification === 'function') {
                this.config.exportNotification(type);
            }
        });
    }
}
TtExportComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtExportComponent, deps: [{ token: TableToolsConfigService }, { token: TableToolsDirective }, { token: DOCUMENT }], target: i0.ɵɵFactoryTarget.Component });
TtExportComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtExportComponent, selector: "tt-export", usesInheritance: true, ngImport: i0, template: "<!--\n  ~ Angular TableTools Plugin\n  ~  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>\n  ~  License: MIT\n  -->\n\n<div>\n    <button class=\"btn btn-outline-primary\" (click)=\"showExport()\">{{lang.export}}</button>\n    <div class=\"modal fade\" [(bsModal)]=\"modal\">\n        <div class=\"modal-dialog modal-lg\">\n            <div class=\"modal-content\">\n                <div class=\"modal-header\">\n                    <h5 class=\"modal-title\">{{lang.export}}</h5>\n                    <button type=\"button\" class=\"close\" dismiss aria-label=\"Close\">\n                        <span aria-hidden=\"true\">&times;</span>\n                    </button>\n                </div>\n                <div class=\"modal-body\">\n                    <div class=\"form-group\">\n                        <label>\n                            <strong>{{lang.exportChooseColumns}}:</strong>\n                            <a href=\"javascript:\" (click)=\"flipSelection()\" class=\"btn btn-link\">\n                                {{lang.flipSelection}}\n                            </a>\n                        </label>\n                        <div>\n                            <div class=\"custom-control custom-checkbox custom-control-inline\"\n                                 *ngFor=\"let c of columns; let index = index\">\n                                <input class=\"custom-control-input\" type=\"checkbox\" id=\"tt-export-{{index}}-{{id}}\"\n                                       [(ngModel)]=\"c.checked\"/>\n                                <label class=\"custom-control-label\" for=\"tt-export-{{index}}-{{id}}\" title=\"{{c.txt}}\">\n                                    {{c.txt}}\n                                </label>\n                            </div>\n                        </div>\n                        <div>\n                            <div class=\"custom-control custom-checkbox mt-2\">\n                                <input class=\"custom-control-input\" type=\"checkbox\" id=\"tt-export-columns-{{id}}\"\n                                       [(ngModel)]=\"exportConfig.columnNames\"/>\n                                <label class=\"custom-control-label\" for=\"tt-export-columns-{{id}}\">\n                                    {{lang.exportColumnNames}}\n                                </label>\n                            </div>\n                        </div>\n                    </div>\n                    <div class=\"form-group\">\n                        <label><strong>{{lang.exportSeparator}}</strong></label>\n                        <div>\n                            <div class=\"custom-control custom-radio custom-control-inline\"\n                                 *ngFor=\"let s of separators; let index = index\">\n                                <input class=\"custom-control-input\" type=\"radio\"\n                                       id=\"tt-export-separator-{{index}}-{{id}}\"\n                                       [(ngModel)]=\"exportConfig.separator\" [value]=\"s.separator\"/>\n                                <label class=\"custom-control-label\" for=\"tt-export-separator-{{index}}-{{id}}\">\n                                    {{s.lang}}\n                                </label>\n                            </div>\n                        </div>\n                    </div>\n                </div>\n                <div class=\"modal-footer\">\n                    <!--suppress TypeScriptValidateTypes -->\n                    <button type=\"button\" class=\"btn btn-outline-primary\"\n                            *ngFor=\"let exportType of exportTypes | keyvalue\"\n                            (click)=\"doExport(exportType.key, exportType.value)\" [disabled]=\"exporting\">\n                        {{exportType.value.lang}}\n                        <span *ngIf=\"exporting == exportType.key\">\n\t\t\t\t\t\t\t<i class=\"fa fa-circle-notch fa-spin\"></i>\n\t\t\t\t\t\t</span>\n                    </button>\n                </div>\n            </div>\n        </div>\n    </div>\n</div>\n", directives: [{ type: i3.BsModalDirective, selector: "[bsModal]", inputs: ["bsModal", "backdrop", "keyboard", "onBeforeChange"], outputs: ["bsModalChange"], exportAs: ["bsModal"] }, { type: i3.DismissDirective, selector: "[dismiss]" }, { type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i1.CheckboxControlValueAccessor, selector: "input[type=checkbox][formControlName],input[type=checkbox][formControl],input[type=checkbox][ngModel]" }, { type: i1.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { type: i1.NgModel, selector: "[ngModel]:not([formControlName]):not([formControl])", inputs: ["name", "disabled", "ngModel", "ngModelOptions"], outputs: ["ngModelChange"], exportAs: ["ngModel"] }, { type: i1.RadioControlValueAccessor, selector: "input[type=radio][formControlName],input[type=radio][formControl],input[type=radio][ngModel]", inputs: ["name", "formControlName", "value"] }, { type: i1.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }], pipes: { "keyvalue": i1$1.KeyValuePipe } });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtExportComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-export',
                    templateUrl: './tt-export.component.html'
                }]
        }], ctorParameters: function () { return [{ type: TableToolsConfigService }, { type: TableToolsDirective }, { type: Document, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }]; } });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtExportModule {
}
TtExportModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtExportModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
TtExportModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtExportModule, declarations: [TtExportComponent], imports: [BsModalModule,
        CommonModule,
        FormsModule], exports: [TtExportComponent] });
TtExportModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtExportModule, imports: [[
            BsModalModule,
            CommonModule,
            FormsModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtExportModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        TtExportComponent
                    ],
                    imports: [
                        BsModalModule,
                        CommonModule,
                        FormsModule
                    ],
                    exports: [
                        TtExportComponent
                    ]
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtResultsCountComponent extends AbstractTableToolsTemplateComponent {
}
TtResultsCountComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtResultsCountComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
TtResultsCountComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtResultsCountComponent, selector: "tt-results-count", usesInheritance: true, ngImport: i0, template: "<!--\n  ~ Angular TableTools Plugin\n  ~  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>\n  ~  License: MIT\n  -->\n<div>\n    {{lang.results}}\n    <span [ngSwitch]=\"tableTools.dataLength\">\n\t\t<span *ngSwitchCase=\"0\">0</span>\n\t\t<span *ngSwitchDefault>\n\t\t\t{{tableTools.pagination.start}} - {{tableTools.pagination.end}}\n            {{lang.from}} {{tableTools.filteredCount}}\n\t\t</span>\n\t</span>\n    <span [hidden]=\"tableTools.filteredCount === tableTools.dataLength\">\n\t\t({{lang.filteredResults}} {{tableTools.dataLength}})\n\t</span>\n</div>\n", directives: [{ type: i1$1.NgSwitch, selector: "[ngSwitch]", inputs: ["ngSwitch"] }, { type: i1$1.NgSwitchCase, selector: "[ngSwitchCase]", inputs: ["ngSwitchCase"] }, { type: i1$1.NgSwitchDefault, selector: "[ngSwitchDefault]" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtResultsCountComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-results-count',
                    templateUrl: './tt-results-count.component.html'
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtPaginationComponent extends AbstractTableToolsTemplateComponent {
}
TtPaginationComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtPaginationComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
TtPaginationComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtPaginationComponent, selector: "tt-pagination", usesInheritance: true, ngImport: i0, template: "<ul class=\"pagination\">\n    <li [ngClass]=\"{disabled: tableTools.pagination.page == 1}\" class=\"page-item\">\n        <a href=\"javascript:\" (click)=\"tableTools.changePage(1)\" title=\"{{lang.first}}\" class=\"page-link\">\n            <i class=\"fa fa-angle-double-left\"></i>\n        </a>\n    </li>\n    <li [ngClass]=\"{disabled: tableTools.pagination.page == 1}\" class=\"page-item\">\n        <a href=\"javascript:\" (click)=\"tableTools.changePage('prev')\" title=\"{{lang.prev}}\" class=\"page-link\">\n            <i class=\"fa fa-angle-left\"></i>\n        </a>\n    </li>\n    <li *ngFor=\"let p of tableTools.pagination.items\" [ngClass]=\"{active: p == tableTools.pagination.page}\"\n        class=\"page-item\">\n        <a href=\"javascript:\" (click)=\"tableTools.changePage(p)\" class=\"page-link\">{{p}}</a>\n    </li>\n    <li [ngClass]=\"{disabled: tableTools.pagination.page == tableTools.pagination.pages\n\t\t|| tableTools.pagination.pages == 0}\" class=\"page-item\">\n        <a href=\"javascript:\" (click)=\"tableTools.changePage('next')\" title=\"{{lang.next}}\" class=\"page-link\">\n            <i class=\"fa fa-angle-right\"></i>\n        </a>\n    </li>\n    <li [ngClass]=\"{disabled: tableTools.pagination.page == tableTools.pagination.pages\n\t\t|| tableTools.pagination.pages == 0}\" class=\"page-item\">\n        <a href=\"javascript:\" (click)=\"tableTools.changePage(tableTools.pagination.pages)\"\n           title=\"{{lang.last}}\" class=\"page-link\">\n            <i class=\"fa fa-angle-double-right\"></i>\n        </a>\n    </li>\n</ul>\n", directives: [{ type: i1$1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtPaginationComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-pagination',
                    templateUrl: './tt-pagination.component.html'
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtPerPageComponent extends AbstractTableToolsTemplateComponent {
}
TtPerPageComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtPerPageComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
TtPerPageComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtPerPageComponent, selector: "tt-per-page", usesInheritance: true, ngImport: i0, template: "<div class=\"form-group\">\n    <label>{{lang.perPage}}&nbsp;</label>\n    <select class=\"form-control\" [formControl]=\"tableTools.perPageControl\">\n        <option *ngFor=\"let o of tableTools.perPageOptions\" [ngValue]=\"o.number\">{{o.text}}</option>\n    </select>\n</div>\n", directives: [{ type: i1.SelectControlValueAccessor, selector: "select:not([multiple])[formControlName],select:not([multiple])[formControl],select:not([multiple])[ngModel]", inputs: ["compareWith"] }, { type: i1.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { type: i1.FormControlDirective, selector: "[formControl]", inputs: ["disabled", "formControl", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { type: i1$1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i1.NgSelectOption, selector: "option", inputs: ["ngValue", "value"] }, { type: i1.ɵNgSelectMultipleOption, selector: "option", inputs: ["ngValue", "value"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtPerPageComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-per-page',
                    templateUrl: './tt-per-page.component.html'
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtPaginationModule {
}
TtPaginationModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtPaginationModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
TtPaginationModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtPaginationModule, declarations: [TtPaginationComponent,
        TtPerPageComponent,
        TtResultsCountComponent], imports: [CommonModule,
        ReactiveFormsModule], exports: [TtPaginationComponent,
        TtPerPageComponent,
        TtResultsCountComponent] });
TtPaginationModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtPaginationModule, imports: [[
            CommonModule,
            ReactiveFormsModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtPaginationModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        TtPaginationComponent,
                        TtPerPageComponent,
                        TtResultsCountComponent
                    ],
                    imports: [
                        CommonModule,
                        ReactiveFormsModule
                    ],
                    exports: [
                        TtPaginationComponent,
                        TtPerPageComponent,
                        TtResultsCountComponent
                    ]
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtPagination {
    constructor(visiblePageCount = 5) {
        this.page = 1;
        this.pages = 1;
        this.start = 0;
        this.end = 0;
        this.items = [];
        this.visiblePageCount = visiblePageCount;
        this.pagesAround = Math.floor(visiblePageCount / 2);
    }
    paginate(resultsLength, perPage) {
        this.pages = Math.ceil(resultsLength / perPage);
        if (this.pages === 0) {
            this.pages = 1;
        }
        if (this.page > this.pages) {
            this.page = this.pages;
        }
        this.items = [];
        const pagesAfter = this.pages - this.page; // number of pages after currently selected page
        let i = this.page // we set a starting page in here
            - (pagesAfter < this.pagesAround // we won't be able to display all pages after current page
                ? this.visiblePageCount - 1 - pagesAfter // so we display the difference before current page
                : this.pagesAround);
        if (i < 1) {
            i = 1;
        }
        do {
            this.items.push(i);
            i++;
        } while (this.items.length < this.visiblePageCount && i <= this.pages);
        this.start = perPage === Infinity
            ? 1
            : Math.min(((this.page - 1) * perPage) + 1, resultsLength);
        this.end = Math.min(this.page * perPage, resultsLength);
    }
}

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtFilterGroup extends FormGroup {
    // eslint-disable-next-line @typescript-eslint/no-useless-constructor
    constructor(filters, validatorOrOpts, asyncValidator) {
        super(filters, validatorOrOpts, asyncValidator);
    }
}

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtFilter extends FormControl {
    constructor(filterOptions, formState, validatorOrOpts, asyncValidator) {
        super(formState, validatorOrOpts, asyncValidator);
        this.filterOptions = {
            empty: '',
            operator: '==',
            or: false
        };
        if (filterOptions) {
            this.filterOptions.field = filterOptions.field;
            if (typeof filterOptions.empty !== 'undefined') {
                this.filterOptions.empty = filterOptions.empty;
            }
            if (typeof filterOptions.operator !== 'undefined') {
                this.filterOptions.operator = filterOptions.operator;
            }
            if (typeof filterOptions.or !== 'undefined') {
                this.filterOptions.or = filterOptions.or;
            }
        }
    }
}

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSearchComponent extends AbstractTableToolsTemplateComponent {
}
TtSearchComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSearchComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
TtSearchComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtSearchComponent, selector: "tt-search", usesInheritance: true, ngImport: i0, template: "<!--\n  ~ Angular TableTools Plugin\n  ~  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>\n  ~  License: MIT\n  -->\n\n<div class=\"form-group\">\n    <input type=\"text\" class=\"form-control\" [formControl]=\"tableTools.searchControl\"\n           [placeholder]=\"lang.search\"/>\n</div>\n", directives: [{ type: i1.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { type: i1.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { type: i1.FormControlDirective, selector: "[formControl]", inputs: ["disabled", "formControl", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSearchComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-search',
                    templateUrl: './tt-search.component.html'
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSearchModule {
}
TtSearchModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSearchModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
TtSearchModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSearchModule, declarations: [TtSearchComponent], imports: [ReactiveFormsModule], exports: [TtSearchComponent] });
TtSearchModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSearchModule, imports: [[
            ReactiveFormsModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSearchModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        TtSearchComponent
                    ],
                    imports: [
                        ReactiveFormsModule
                    ],
                    exports: [
                        TtSearchComponent
                    ]
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSearchService {
    getFilterFn(search, filters) {
        const hasSearch = typeof search === 'string' && search !== '', fieldFilters = this.getFieldFilters(filters), hasFilters = Object.keys(fieldFilters).length;
        return (value) => {
            if (!hasSearch && !hasFilters) {
                return true;
            }
            return (!hasSearch || this.hasSearchString(value, search))
                && (!hasFilters || typeof value !== 'object' || value === null
                    || this.areFiltersValid(value, fieldFilters));
        };
    }
    hasSearchString(variable, search) {
        if (typeof variable === 'object' && variable !== null) {
            return !!Object.keys(variable).find((key) => {
                return this.hasSearchString(variable[key], search);
            });
        }
        else if ((typeof variable === 'number' && variable.toString() === search)
            || (typeof variable === 'string' && variable.toLowerCase().includes(search))) {
            return true;
        }
        return false;
    }
    getFieldFilters(filters) {
        if (typeof filters === 'undefined') {
            return {};
        }
        const fieldFilters = {};
        Object.entries(filters.controls)
            .filter(([, filter]) => {
            return typeof filter.value !== 'undefined'
                && filter.value !== null
                && filter.value !== filter.filterOptions.empty
                && !(Array.isArray(filter.value) && !filter.value.length);
        })
            .forEach(([field, filter]) => {
            if (!filter.filterOptions.field) {
                filter.filterOptions.field = field;
            }
            const fieldName = filter.filterOptions.field || field;
            if (typeof fieldFilters[fieldName] === 'undefined') {
                fieldFilters[fieldName] = {
                    or: [],
                    and: []
                };
            }
            const filterVal = { operator: filter.filterOptions.operator, value: filter.value };
            if (filter.filterOptions.or) {
                fieldFilters[fieldName].or.push(filterVal);
            }
            else {
                fieldFilters[fieldName].and.push(filterVal);
            }
        });
        return fieldFilters;
    }
    areFiltersValid(item, fieldFilters) {
        return Object.entries(fieldFilters)
            .every(([field, filters]) => {
            const compareFn = this.getCompareFn(item[field]);
            return (!filters.or.length || filters.or.some(compareFn)) // [].some() = false
                && filters.and.every(compareFn); // [].every() = true
        });
    }
    getCompareFn(value) {
        return (filter) => {
            return this.compareWithOperator(value, filter.value, filter.operator);
        };
    }
    compareWithOperator(variable, search, operator) {
        if (typeof search === 'object') {
            return !!Object.values(search)
                .find((value) => this.compareWithOperator(variable, value, operator));
        }
        if (typeof variable === 'object') {
            return !!Object.keys(variable)
                .find((key) => this.compareWithOperator(variable[key], search, operator));
        }
        if (operator === 'like') {
            return variable.toLowerCase().includes(search.toLowerCase());
        }
        else {
            switch (operator) {
                case '>':
                    // @ts-expect-error: unknown comparison
                    return variable > search;
                case '<':
                    // @ts-expect-error: unknown comparison
                    return variable < search;
                case '>=':
                    // @ts-expect-error: unknown comparison
                    return variable >= search;
                case '<=':
                    // @ts-expect-error: unknown comparison
                    return variable <= search;
                case '==':
                    // eslint-disable-next-line eqeqeq
                    return variable == search;
                default:
                    return true;
            }
        }
    }
}
TtSearchService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSearchService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
TtSearchService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSearchService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSearchService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSelectAllComponent {
    constructor(tableToolsDirective) {
        this.tableToolsDirective = tableToolsDirective;
        this.formControl = new FormControl();
    }
    ngOnInit() {
        this.formControl.valueChanges.subscribe((selected) => {
            if (selected) {
                this.tableToolsDirective.tableTools.selected.select(...this.tableToolsDirective.ttSelects.map((ttSelect) => ttSelect.item));
            }
            else {
                this.tableToolsDirective.tableTools.selected.clear();
            }
        });
        this.tableToolsDirective.tableTools.selected.selectedChanges.subscribe((selected) => {
            const all = selected.length === this.tableToolsDirective.ttSelects.length;
            if (all !== !!this.formControl.value) {
                this.formControl.setValue(all, { emitEvent: false });
            }
        });
    }
}
TtSelectAllComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSelectAllComponent, deps: [{ token: TableToolsDirective, host: true }], target: i0.ɵɵFactoryTarget.Component });
TtSelectAllComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtSelectAllComponent, selector: "tt-select-all", ngImport: i0, template: "<!--\n  ~ Angular TableTools Plugin\n  ~  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>\n  ~  License: MIT\n  -->\n<input type=\"checkbox\" [formControl]=\"formControl\"/>\n", directives: [{ type: i1.CheckboxControlValueAccessor, selector: "input[type=checkbox][formControlName],input[type=checkbox][formControl],input[type=checkbox][ngModel]" }, { type: i1.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { type: i1.FormControlDirective, selector: "[formControl]", inputs: ["disabled", "formControl", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSelectAllComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-select-all',
                    templateUrl: './tt-select.component.html'
                }]
        }], ctorParameters: function () { return [{ type: TableToolsDirective, decorators: [{
                    type: Host
                }] }]; } });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSelectModule {
}
TtSelectModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSelectModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
TtSelectModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSelectModule, declarations: [TtSelectAllComponent,
        TtSelectComponent], imports: [ReactiveFormsModule], exports: [TtSelectAllComponent,
        TtSelectComponent] });
TtSelectModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSelectModule, imports: [[
            ReactiveFormsModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSelectModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        ReactiveFormsModule
                    ],
                    declarations: [
                        TtSelectAllComponent,
                        TtSelectComponent
                    ],
                    exports: [
                        TtSelectAllComponent,
                        TtSelectComponent
                    ]
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSelect {
    constructor() {
        this.selected = new Map();
        this.selectedChanges = new Subject();
    }
    getSelected() {
        return [...this.selected.keys()];
    }
    hasSelected() {
        return this.selected.size > 0;
    }
    select(...items) {
        const nonSelected = items.filter((item) => !this.selected.has(item));
        if (!nonSelected.length) {
            return;
        }
        nonSelected.forEach((item) => {
            this.selected.set(item, true);
        });
        this.selectedChanges.next(this.getSelected());
    }
    deselect(...items) {
        const selected = items.filter((item) => this.selected.has(item));
        if (!selected.length) {
            return;
        }
        selected.forEach((item) => {
            this.selected.delete(item);
        });
        this.selectedChanges.next(this.getSelected());
    }
    clear() {
        if (!this.hasSelected()) {
            return;
        }
        this.selected.clear();
        this.selectedChanges.next(this.getSelected());
    }
}

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSortDirective {
    constructor(tableToolsDirective) {
        this.tableToolsDirective = tableToolsDirective;
    }
    get sort() {
        return typeof this.ttSort !== 'undefined';
    }
    get isAsc() {
        return this.state === 'asc';
    }
    get isDesc() {
        return this.state === 'desc';
    }
    click(shiftKey) {
        if (typeof this.ttSort === 'undefined') {
            return;
        }
        const orderItem = this.getMyOrder(this.tableTools.order), newState = (orderItem === null || orderItem === void 0 ? void 0 : orderItem.direction) === 'asc' ? 'desc' : 'asc';
        if (!shiftKey) {
            if ((orderItem === null || orderItem === void 0 ? void 0 : orderItem.direction) === 'desc') {
                this.tableTools.order.splice(this.tableTools.order.indexOf(orderItem), 1);
                this.tableTools.order = [...this.tableTools.order];
                return;
            }
            this.tableTools.order = [{ field: this.ttSort, direction: newState }];
        }
        else {
            if (!orderItem) {
                this.tableTools.order.push({ field: this.ttSort, direction: newState });
            }
            else {
                orderItem.direction = orderItem.direction === 'asc' ? 'desc' : 'asc';
            }
            this.tableTools.order = [...this.tableTools.order];
        }
    }
    ngOnInit() {
        this.tableTools = this.tableToolsDirective.tableTools;
        this.tableTools.orderChanges.subscribe((order) => {
            var _a;
            this.state = (_a = this.getMyOrder(order)) === null || _a === void 0 ? void 0 : _a.direction;
        });
    }
    getMyOrder(order) {
        return order
            .find((x) => x.field === this.ttSort);
    }
}
TtSortDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSortDirective, deps: [{ token: TableToolsDirective, host: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.Directive });
TtSortDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.0.5", type: TtSortDirective, selector: "[ttSort]", inputs: { ttSort: "ttSort" }, host: { listeners: { "click": "click($event.shiftKey)" }, properties: { "class.tt-sort": "this.sort", "class.sorting-asc": "this.isAsc", "class.sorting-desc": "this.isDesc" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSortDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ttSort]'
                }]
        }], ctorParameters: function () { return [{ type: TableToolsDirective, decorators: [{
                    type: SkipSelf
                }, {
                    type: Host
                }] }]; }, propDecorators: { ttSort: [{
                type: Input
            }], sort: [{
                type: HostBinding,
                args: ['class.tt-sort']
            }], isAsc: [{
                type: HostBinding,
                args: ['class.sorting-asc']
            }], isDesc: [{
                type: HostBinding,
                args: ['class.sorting-desc']
            }], click: [{
                type: HostListener,
                args: ['click', ['$event.shiftKey']]
            }] } });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSortModule {
}
TtSortModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSortModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
TtSortModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSortModule, declarations: [TtSortDirective], exports: [TtSortDirective] });
TtSortModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSortModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSortModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        TtSortDirective
                    ],
                    exports: [
                        TtSortDirective
                    ]
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtSortService {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/explicit-module-boundary-types
    getSortFn(order) {
        return (a, b) => {
            if (!order.length) {
                return 0;
            }
            let result = 0;
            order.some((orderItem) => {
                result = this.compareFn(a[orderItem.field], b[orderItem.field]);
                if (orderItem.direction === 'desc' && result !== 0) {
                    result = result === -1 ? 1 : -1;
                }
                return result !== 0;
            });
            return result;
        };
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/explicit-module-boundary-types
    compareFn(a, b) {
        const typeA = typeof a, typeB = typeof b;
        if (typeA !== typeB) {
            return typeA === 'undefined' ? 1
                : typeB === 'undefined' ? -1
                    : a === null ? 1
                        : b === null ? -1
                            : typeA < typeB ? -1 : 1;
        }
        if (typeof a === 'string' && typeof b === 'string') {
            if (this.isNumeric(a) && this.isNumeric(b)) {
                return parseFloat(a) < parseFloat(b) ? -1 : 1;
            }
            // Compare strings case-insensitively
            a = a.toLowerCase();
            b = b.toLowerCase();
        }
        if (a !== b) {
            if (typeof a.localeCompare === 'function') {
                return a.localeCompare(b);
            }
            return a < b ? -1 : 1;
        }
        return 0;
    }
    isNumeric(string) {
        const n = parseFloat(string);
        return !isNaN(n) && isFinite(n);
    }
}
TtSortService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSortService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
TtSortService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSortService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtSortService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtLoadingComponent extends AbstractTableToolsTemplateComponent {
}
TtLoadingComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtLoadingComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
TtLoadingComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtLoadingComponent, selector: "tt-loading", inputs: { extraCondition: "extraCondition" }, usesInheritance: true, ngImport: i0, template: "<span [hidden]=\"!tableTools.loading && !extraCondition\">&nbsp;<i class=\"fa fa-circle-notch fa-spin fa-lg\"></i></span>\n" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtLoadingComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-loading',
                    templateUrl: './tt-loading.component.html'
                }]
        }], propDecorators: { extraCondition: [{
                type: Input
            }] } });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtHeaderComponent {
}
TtHeaderComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtHeaderComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
TtHeaderComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtHeaderComponent, selector: "tt-header", ngImport: i0, template: "<!--\n  ~ Angular TableTools Plugin\n  ~  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>\n  ~  License: MIT\n  -->\n<div class=\"form-inline\">\n    <tt-per-page></tt-per-page>\n    <tt-loading></tt-loading>\n    <tt-search class=\"ml-auto\"></tt-search>\n</div>\n<div class=\"row mt-3\">\n    <tt-results-count class=\"col align-self-center\"></tt-results-count>\n    <tt-pagination class=\"col col-auto pr-0\"></tt-pagination>\n    <tt-export class=\"col col-auto pl-2\"></tt-export>\n</div>\n", components: [{ type: TtPerPageComponent, selector: "tt-per-page" }, { type: TtLoadingComponent, selector: "tt-loading", inputs: ["extraCondition"] }, { type: TtSearchComponent, selector: "tt-search" }, { type: TtResultsCountComponent, selector: "tt-results-count" }, { type: TtPaginationComponent, selector: "tt-pagination" }, { type: TtExportComponent, selector: "tt-export" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtHeaderComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-header',
                    templateUrl: './tt-header.component.html'
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtFooterComponent {
}
TtFooterComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtFooterComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
TtFooterComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtFooterComponent, selector: "tt-footer", ngImport: i0, template: "<!--\n  ~ Angular TableTools Plugin\n  ~  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>\n  ~  License: MIT\n  -->\n\n<div class=\"row\">\n    <tt-results-count class=\"col align-self-center\"></tt-results-count>\n    <tt-pagination class=\"col col-auto\"></tt-pagination>\n</div>\n", components: [{ type: TtResultsCountComponent, selector: "tt-results-count" }, { type: TtPaginationComponent, selector: "tt-pagination" }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtFooterComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'tt-footer',
                    templateUrl: './tt-footer.component.html'
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TtRowPlaceholderComponent extends AbstractTableToolsTemplateComponent {
}
TtRowPlaceholderComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtRowPlaceholderComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
TtRowPlaceholderComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.0.5", type: TtRowPlaceholderComponent, selector: "[ttRowPlaceholder]", usesInheritance: true, ngImport: i0, template: "<!--\n  ~ Angular TableTools Plugin\n  ~  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>\n  ~  License: MIT\n  -->\n\n<td colspan=\"100%\" *ngIf=\"!(tableTools.data | async)?.length\">\n    <tt-loading></tt-loading>\n    <span *ngIf=\"!tableTools.loading\">{{lang.noResults}}</span>\n</td>\n", components: [{ type: TtLoadingComponent, selector: "tt-loading", inputs: ["extraCondition"] }], directives: [{ type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }], pipes: { "async": i1$1.AsyncPipe } });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TtRowPlaceholderComponent, decorators: [{
            type: Component,
            args: [{
                    selector: '[ttRowPlaceholder]',
                    templateUrl: './tt-row-placeholder.component.html'
                }]
        }] });

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */
class TableToolsModule {
}
TableToolsModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
TableToolsModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsModule, declarations: [TableToolsDirective,
        TtFooterComponent,
        TtHeaderComponent,
        TtLoadingComponent,
        TtRowPlaceholderComponent], imports: [CommonModule,
        TtExportModule,
        TtPaginationModule,
        TtSearchModule,
        TtSelectModule,
        TtSortModule], exports: [TableToolsDirective,
        TtExportModule,
        TtFooterComponent,
        TtHeaderComponent,
        TtLoadingComponent,
        TtPaginationModule,
        TtRowPlaceholderComponent,
        TtSearchModule,
        TtSelectModule,
        TtSortModule] });
TableToolsModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsModule, imports: [[
            CommonModule,
            TtExportModule,
            TtPaginationModule,
            TtSearchModule,
            TtSelectModule,
            TtSortModule
        ], TtExportModule,
        TtPaginationModule,
        TtSearchModule,
        TtSelectModule,
        TtSortModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        CommonModule,
                        TtExportModule,
                        TtPaginationModule,
                        TtSearchModule,
                        TtSelectModule,
                        TtSortModule
                    ],
                    declarations: [
                        TableToolsDirective,
                        TtFooterComponent,
                        TtHeaderComponent,
                        TtLoadingComponent,
                        TtRowPlaceholderComponent
                    ],
                    exports: [
                        TableToolsDirective,
                        TtExportModule,
                        TtFooterComponent,
                        TtHeaderComponent,
                        TtLoadingComponent,
                        TtPaginationModule,
                        TtRowPlaceholderComponent,
                        TtSearchModule,
                        TtSelectModule,
                        TtSortModule
                    ]
                }]
        }] });

class TableToolsService {
    constructor(config, document, ttSort, ttSearch) {
        this.config = config;
        this.document = document;
        this.ttSort = ttSort;
        this.ttSearch = ttSearch;
    }
    create(options) {
        return new TableTools(options, this.config, this.document, this.ttSort, this.ttSearch);
    }
}
TableToolsService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsService, deps: [{ token: TableToolsConfigService }, { token: DOCUMENT }, { token: TtSortService }, { token: TtSearchService }], target: i0.ɵɵFactoryTarget.Injectable });
TableToolsService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.0.5", ngImport: i0, type: TableToolsService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: TableToolsConfigService }, { type: Document, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: TtSortService }, { type: TtSearchService }]; } });
class TableTools {
    constructor(options, config, document, ttSort, ttSearch) {
        this.document = document;
        this.ttSort = ttSort;
        this.ttSearch = ttSearch;
        this.loading = false;
        this.data = new BehaviorSubject([]);
        this.dataLength = 0;
        this.filteredCount = 0;
        // region search
        this.searchControl = new FormControl();
        // endregion select
        this.lastResolve = {
            subscription: null,
            timeout: null
        };
        this.configuring = false;
        this.searchControl.valueChanges.subscribe(() => {
            this.filterData();
        });
        this.filters = options.filters || new TtFilterGroup({});
        this.filters.valueChanges.subscribe(() => {
            this.filterData();
        });
        this.perPageControl = new FormControl(options.perPage || config.perPage);
        this.perPageOptions = options.perPageOptions || config.perPageOptions;
        this.perPageControl.valueChanges.subscribe(() => {
            this.filterData();
        });
        this.scroll = typeof options.scroll !== 'undefined' ? options.scroll : config.scroll;
        this.scrollOffset = typeof options.scrollOffset !== 'undefined' ? options.scrollOffset : 0;
        this.orderChanges = new BehaviorSubject(options.order || []);
        this.orderChanges
            .pipe(skip(1))
            .subscribe(() => {
            this.filterData();
        });
        this.selected = new TtSelect();
        this.pagination = new TtPagination();
        this.collection = options.collection;
        if (typeof options.resolver !== 'undefined') {
            this.resolver = options.resolver;
        }
        if (typeof options.url !== 'undefined') {
            this.url = options.url;
            this.resolver = this.resolver || config.defaultTableToolsResolver;
        }
        if (typeof options.asyncConfigurator === 'function') {
            this.configuring = true;
        }
        setTimeout(() => {
            if (typeof options.asyncConfigurator === 'function') {
                options.asyncConfigurator(this)
                    .pipe(first())
                    .subscribe(() => {
                    this.configuring = false;
                    this.filterData();
                });
            }
            else {
                this.filterData();
            }
        });
    }
    get perPage() {
        return this.perPageControl.value;
    }
    set perPage(value) {
        if (this.perPageControl.value !== value) {
            this.perPageControl.setValue(value);
        }
    }
    get order() {
        return this.orderChanges.getValue();
    }
    set order(order) {
        this.orderChanges.next(order);
    }
    filterData() {
        if (this.configuring) {
            return;
        }
        if (typeof this.resolver !== 'undefined') {
            this.serverSide();
        }
        else {
            this.browserSide();
        }
    }
    changePage(page) {
        var _a, _b;
        const originalPage = this.pagination.page;
        if (page === 'prev') {
            if (this.pagination.page > 1) {
                this.pagination.page--;
            }
        }
        else if (page === 'next') {
            if (this.pagination.page < this.pagination.pages) {
                this.pagination.page++;
            }
        }
        else if (!isNaN(page)) {
            this.pagination.page = page;
        }
        if (originalPage !== this.pagination.page) {
            this.filterData();
        }
        if (this.scroll) {
            this.scrollTo(Math.round((((_a = this.elementRef) === null || _a === void 0 ? void 0 : _a.nativeElement.getBoundingClientRect().top) || 0)
                + (((_b = this.document.defaultView) === null || _b === void 0 ? void 0 : _b.pageYOffset) || this.document.documentElement.scrollTop)) + this.scrollOffset, 1000);
        }
    }
    serverSide() {
        let timeout = 0;
        if (this.lastResolve.timeout !== null) {
            clearTimeout(this.lastResolve.timeout);
            if (this.lastResolve.subscription !== null) {
                this.lastResolve.subscription.unsubscribe();
            }
            timeout = 750;
        }
        this.lastResolve.timeout = setTimeout(() => {
            this.lastResolve.subscription = this.resolver(this.perPage, (this.pagination.page - 1) * this.perPage, this.order, typeof this.searchControl.value !== 'string' ? '' : this.searchControl.value, this.ttSearch.getFieldFilters(this.filters), this.url).subscribe((result) => {
                if (typeof result.data === 'undefined'
                    || typeof result.count !== 'number'
                    || typeof result.countFiltered !== 'number') {
                    throw new Error('TableTools - wrong result format');
                }
                this.data.next(result.data);
                this.dataLength = result.count;
                this.filteredCount = result.countFiltered;
                if (this.pagination.page > 1 && !result.data.length) {
                    this.changePage(1);
                }
            }, (e) => {
                console.error(e);
                this.data.next([]);
                this.dataLength = 0;
                this.filteredCount = 0;
            }, () => {
                this.pagination.paginate(this.filteredCount, this.perPage);
                this.selected.clear();
                this.loading = false;
                this.lastResolve.timeout = null;
                this.lastResolve.subscription = null;
            });
        }, timeout);
    }
    browserSide() {
        this.loading = true;
        let timeout = 0;
        if (this.lastResolve.timeout !== null) {
            clearTimeout(this.lastResolve.timeout);
            timeout = 50;
        }
        this.lastResolve.timeout = setTimeout(() => {
            var _a, _b;
            this.dataLength = ((_a = this.collection) === null || _a === void 0 ? void 0 : _a.length) || 0;
            let data = this.collection
                ? this.collection.filter(this.ttSearch.getFilterFn((_b = this.searchControl.value) === null || _b === void 0 ? void 0 : _b.toLowerCase(), this.filters))
                : [];
            this.filteredCount = data.length;
            if (this.order.length) {
                data.sort(this.ttSort.getSortFn(this.order));
            }
            this.pagination.paginate(data.length, this.perPage);
            data = data.slice(this.pagination.start - 1, this.pagination.end);
            this.selected.clear();
            this.lastResolve.timeout = null;
            this.loading = false;
            this.data.next(data);
        }, timeout);
    }
    scrollTo(target, duration) {
        var _a;
        const cur = window.scrollY, start = performance.now(), step = (ts) => {
            var _a, _b, _c;
            const elapsed = ts - start;
            if (elapsed >= 1000) {
                (_a = this.document.defaultView) === null || _a === void 0 ? void 0 : _a.scrollTo(0, target);
                return;
            }
            (_b = this.document.defaultView) === null || _b === void 0 ? void 0 : _b.scrollTo(0, cur - Math.sin((Math.PI / 2) / (duration / elapsed)) * (cur - target));
            (_c = this.document.defaultView) === null || _c === void 0 ? void 0 : _c.requestAnimationFrame(step);
        };
        (_a = this.document.defaultView) === null || _a === void 0 ? void 0 : _a.requestAnimationFrame(step);
    }
}

/*
 * Angular TableTools Plugin
 *  Copyright (c) 2016-2021 Rodziu <mateusz.rohde@gmail.com>
 *  License: MIT
 */

/**
 * Generated bundle index. Do not edit.
 */

export { TableToolsConfigService, TableToolsDirective, TableToolsModule, TableToolsService, TtExportComponent, TtExportModule, TtFilter, TtFilterGroup, TtFooterComponent, TtHeaderComponent, TtLoadingComponent, TtPagination, TtPaginationComponent, TtPaginationModule, TtPerPageComponent, TtResultsCountComponent, TtRowPlaceholderComponent, TtSearchComponent, TtSearchModule, TtSearchService, TtSelect, TtSelectAllComponent, TtSelectComponent, TtSelectModule, TtSortDirective, TtSortModule, TtSortService };
//# sourceMappingURL=angular-bootstrap4-table-tools.js.map
