import { IOrder, IPerPageOption, TableToolsConfigService, TableToolsResolver } from './table-tools-config.service';
import { ElementRef } from '@angular/core';
import { TtPagination } from './pagination/tt-pagination';
import { BehaviorSubject, Observable } from 'rxjs';
import { TtSortService } from './sort/tt-sort.service';
import { FormControl } from '@angular/forms';
import { TtSearchService } from './search/tt-search.service';
import { TtFilterGroup } from './search/tt-filter-group';
import { TtSelect } from './select/tt-select';
import * as i0 from "@angular/core";
export interface ITableToolsOptions<T extends object> {
    perPage: number;
    perPageOptions: IPerPageOption[];
    scroll: boolean;
    scrollOffset?: number;
    order: IOrder[];
    filters: TtFilterGroup;
    collection?: T[];
    url?: string;
    resolver?: TableToolsResolver<T>;
    asyncConfigurator?: (tableInstance: ITableTools<T>) => Observable<void>;
}
export interface ITableTools<T extends object> extends ITableToolsOptions<T> {
    loading: boolean;
    data: BehaviorSubject<T[]>;
    dataLength: number;
    filteredCount: number;
    searchControl: FormControl;
    perPageControl: FormControl;
    pagination: TtPagination;
    selected: TtSelect<T>;
    elementRef?: ElementRef<HTMLElement>;
    orderChanges: BehaviorSubject<IOrder[]>;
    filterData(): void;
    changePage(page: number | 'prev' | 'next'): void;
}
export declare class TableToolsService {
    private config;
    private document;
    private ttSort;
    private ttSearch;
    constructor(config: TableToolsConfigService, document: Document, ttSort: TtSortService, ttSearch: TtSearchService);
    create<T extends object>(options: Partial<ITableToolsOptions<T>>): ITableTools<T>;
    static ɵfac: i0.ɵɵFactoryDeclaration<TableToolsService, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<TableToolsService>;
}
